==> open-xchange-documents-backend_7.10.3.orig.tar.bz2: open-xchange-documents-backend-7.10.3/com.openexchange.office.rt2.core/src/com/openexchange/office/rt2/core/metric/WebsocketResponseMetricService.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.rt2.core.metric;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.codahale.metrics.Gauge;
import com.codahale.metrics.MetricRegistry;
import com.openexchange.exception.ExceptionUtils;
import com.openexchange.office.rt2.protocol.value.RT2MessageIdType;
import com.openexchange.office.tools.osgi.ServiceLookupRegistry;

public class WebsocketResponseMetricService extends BaseMetricService {
	private static final Logger log = LoggerFactory.getLogger(WebsocketResponseMetricService.class);
	private static final long MSG_TIME_CLEANUP_FREQUENCY_IN_MS = 1000 * 60 * 10;
	private static final long MSG_TMEOUT_FOR_CLEANUP_IN_MINS   = 3;

	private static class MessageTimeEntry implements Comparable<MessageTimeEntry> {
		public final RT2MessageIdType id;
		public final LocalDateTime dateTime;

		public MessageTimeEntry(RT2MessageIdType id) {
			this.id = id;
			this.dateTime = LocalDateTime.now();
		}

		@Override
		public int compareTo(MessageTimeEntry o) {
			return this.dateTime.compareTo(o.dateTime);
		}
	}

	private final Map<RT2MessageIdType, MessageTimeEntry> receivedMessageIds = Collections.synchronizedMap(new HashMap<>());
	private final Thread receivedMsgCleanupThread;

	private AtomicLong messagesReceived = new AtomicLong(0);
	private AtomicLong succMessages = new AtomicLong(0);
	private AtomicLong failedMessages = new AtomicLong(0);

	public WebsocketResponseMetricService() {
		super(MetricRegistry.name("WSChannel", "runtime"));
		MetricRegistry metricRegistry = ServiceLookupRegistry.get().getService(MetricRegistry.class);
		metricRegistry.register(MetricRegistry.name("messages", "countAll"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return messagesReceived.get();
					}
				});
		metricRegistry.register(MetricRegistry.name("messages", "countSucc"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return succMessages.get();
					}
				});
		metricRegistry.register(MetricRegistry.name("messages", "countFailed"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return failedMessages.get();
					}
				});

		this.receivedMsgCleanupThread = new Thread(new ReceivedMsgIdsCleanupTask());
		receivedMsgCleanupThread.setName("ReceivedMsgIds.CleanupThread");
		receivedMsgCleanupThread.setDaemon(true);
		receivedMsgCleanupThread.start();
	}

	public void incMessagesReceived(RT2MessageIdType msgId) {
		receivedMessageIds.put(msgId, new MessageTimeEntry(msgId));
		messagesReceived.incrementAndGet();
	}

	public void incMessageSucc(RT2MessageIdType msgId) {
		if (receivedMessageIds.containsKey(msgId)) {
			succMessages.incrementAndGet();
			receivedMessageIds.remove(msgId);
		}
	}

	public void incMessageFailed(RT2MessageIdType msgId) {
		if (receivedMessageIds.containsKey(msgId)) {
			failedMessages.incrementAndGet();
			receivedMessageIds.remove(msgId);
		}
	}

	private class ReceivedMsgIdsCleanupTask implements Runnable {

		@Override
		public void run() {
			while (true) {
				try {
					Thread.sleep(MSG_TIME_CLEANUP_FREQUENCY_IN_MS);
					final LocalDateTime cmpDt = LocalDateTime.now().minusMinutes(MSG_TMEOUT_FOR_CLEANUP_IN_MINS);
					final AtomicLong removed = new AtomicLong(0);

					synchronized (receivedMessageIds) {
						List<MessageTimeEntry> timeList = new ArrayList<MessageTimeEntry>(receivedMessageIds.values()); 
						Collections.sort(timeList);
						for (MessageTimeEntry dt : timeList) {
							if (dt.dateTime.isBefore(cmpDt)) {
								receivedMessageIds.remove(dt.id);
								removed.incrementAndGet();
							}
						}
					}

					log.debug("{} removed {} entries from receivedMessageIds map - map has still {} entries", Thread.currentThread().getName(), removed.get(), receivedMessageIds.size());
				} catch (Throwable ex) {
					ExceptionUtils.handleThrowable(ex);
					log.warn("CleanupTask: Exception caught while trying to remove old metric data", ex);
				}
			}
		}

	}

}

==> open-xchange-documents-backend_7.10.4.orig.tar.bz2: open-xchange-documents-backend-7.10.4/com.openexchange.office.rt2.core/src/com/openexchange/office/rt2/core/metric/WebsocketResponseMetricService.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.rt2.core.metric;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.codahale.metrics.Gauge;
import com.codahale.metrics.MetricRegistry;
import com.openexchange.exception.ExceptionUtils;
import com.openexchange.office.rt2.protocol.value.RT2MessageIdType;

@Service
public class WebsocketResponseMetricService extends BaseMetricService implements InitializingBean, DisposableBean{
	
	private static final Logger log = LoggerFactory.getLogger(WebsocketResponseMetricService.class);
	
	private static final long MSG_TIME_CLEANUP_FREQUENCY_IN_MS = 1000 * 60 * 10;
	private static final long MSG_TMEOUT_FOR_CLEANUP_IN_MINS   = 3;

	private static class MessageTimeEntry implements Comparable<MessageTimeEntry> {
		public final RT2MessageIdType id;
		public final LocalDateTime dateTime;

		public MessageTimeEntry(RT2MessageIdType id) {
			this.id = id;
			this.dateTime = LocalDateTime.now();
		}

		@Override
		public int compareTo(MessageTimeEntry o) {
			return this.dateTime.compareTo(o.dateTime);
		}
	}

	@Autowired
	private MetricRegistry metricRegistry;
	
	private final Map<RT2MessageIdType, MessageTimeEntry> receivedMessageIds = Collections.synchronizedMap(new HashMap<>());
	private Thread receivedMsgCleanupThread;

	private AtomicLong messagesReceived = new AtomicLong(0);
	private AtomicLong succMessages = new AtomicLong(0);
	private AtomicLong failedMessages = new AtomicLong(0);

	private AtomicBoolean stopped = new AtomicBoolean(false);
	
	public WebsocketResponseMetricService() {
		super(MetricRegistry.name("WSChannel", "runtime"));
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		super.afterPropertiesSet();
		metricRegistry.register(MetricRegistry.name("messages", "countAll"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return messagesReceived.get();
					}
				});
		metricRegistry.register(MetricRegistry.name("messages", "countSucc"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return succMessages.get();
					}
				});
		metricRegistry.register(MetricRegistry.name("messages", "countFailed"), 
				new Gauge<Long>() {
					@Override
					public Long getValue() {
						return failedMessages.get();
					}
				});

		this.receivedMsgCleanupThread = new Thread(new ReceivedMsgIdsCleanupTask());
		receivedMsgCleanupThread.setName("ReceivedMsgIds.CleanupThread");
		receivedMsgCleanupThread.setDaemon(true);
		receivedMsgCleanupThread.start();
	}

	@Override
	public void destroy() throws Exception {
		stopped.set(true);
	}

	public void incMessagesReceived(RT2MessageIdType msgId) {
		receivedMessageIds.put(msgId, new MessageTimeEntry(msgId));
		messagesReceived.incrementAndGet();
	}

	public void incMessageSucc(RT2MessageIdType msgId) {
		if (receivedMessageIds.containsKey(msgId)) {
			succMessages.incrementAndGet();
			receivedMessageIds.remove(msgId);
		}
	}

	public void incMessageFailed(RT2MessageIdType msgId) {
		if (receivedMessageIds.containsKey(msgId)) {
			failedMessages.incrementAndGet();
			receivedMessageIds.remove(msgId);
		}
	}

	private class ReceivedMsgIdsCleanupTask implements Runnable {

		@Override
		public void run() {
			while (!stopped.get()) {
				try {
					Thread.sleep(MSG_TIME_CLEANUP_FREQUENCY_IN_MS);
					final LocalDateTime cmpDt = LocalDateTime.now().minusMinutes(MSG_TMEOUT_FOR_CLEANUP_IN_MINS);
					final AtomicLong removed = new AtomicLong(0);

					synchronized (receivedMessageIds) {
						List<MessageTimeEntry> timeList = new ArrayList<MessageTimeEntry>(receivedMessageIds.values()); 
						Collections.sort(timeList);
						for (MessageTimeEntry dt : timeList) {
							if (dt.dateTime.isBefore(cmpDt)) {
								receivedMessageIds.remove(dt.id);
								removed.incrementAndGet();
							}
						}
					}

					log.debug("{} removed {} entries from receivedMessageIds map - map has still {} entries", Thread.currentThread().getName(), removed.get(), receivedMessageIds.size());
				} catch (Throwable ex) {
					ExceptionUtils.handleThrowable(ex);
					log.warn("CleanupTask: Exception caught while trying to remove old metric data", ex);
				}
			}
		}

	}

}
